@@ -1,13 +1,14 @@
 
-scam.check <- function(b)
+scam.check <- function(b,rl.col=3,...)
 # takes a fitted scam object and produces some standard diagnostic plots
 {   old.par<-par(mfrow=c(2,2))
     sc.name<-b$method
-    qqnorm(residuals(b))
+    qqnorm(residuals(b),...)
+    qqline(residuals(b),col=rl.col,...)
     plot(b$linear.predictors,residuals(b),main="Resids vs. linear pred.",
-         xlab="linear predictor",ylab="residuals");
-    hist(residuals(b),xlab="Residuals",main="Histogram of residuals");
-    plot(fitted(b),b$y,xlab="Fitted Values",ylab="Response",main="Response vs. Fitted Values")
+         xlab="linear predictor",ylab="residuals",...);
+    hist(residuals(b),xlab="Residuals",main="Histogram of residuals",...);
+    plot(fitted(b),b$y,xlab="Fitted Values",ylab="Response",main="Response vs. Fitted Values",...)
     
     ## now summarize convergence information 
     cat("\nMethod:", b$method, "  Optimizer:", b$optimizer)
